@@ -104,7 +104,7 @@ sub parse {
 
   # Relaxed parsing
   my $headers = $self->headers;
-  my $len = $headers->content_length // '';
+  my $len     = $headers->content_length // '';
   if ($self->auto_relax && !length $len) {
     my $connection = lc($headers->connection // '');
     $self->relaxed(1)
@@ -594,6 +594,6 @@ time to end the stream.
 
 =head1 SEE ALSO
 
-L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
+L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.
 
 =cut
